﻿#nullable enable

using System;
using System.Threading;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;

namespace Unity.Extensions.GenericHost.Internal
{
    internal class UnityApplicationLifetime : IHostApplicationLifetime
    {
        private readonly CancellationTokenSource _startedSource = new();
        private readonly CancellationTokenSource _stoppingSource = new();
        private readonly CancellationTokenSource _stoppedSource = new();
        private readonly ILogger<UnityApplicationLifetime> _logger;

        public CancellationToken ApplicationStarted => _startedSource.Token;
        public CancellationToken ApplicationStopping => _stoppingSource.Token;
        public CancellationToken ApplicationStopped => _stoppedSource.Token;
        
        public UnityApplicationLifetime(ILogger<UnityApplicationLifetime> logger)
        {
            _logger = logger;
        }
        
        public void StopApplication()
        {
            lock (_stoppingSource)
            {
                try
                {
                    _stoppingSource.Cancel();
                }
                catch (Exception e)
                {
                    _logger.LogCritical(
                        eventId: LoggerEventIds.ApplicationStoppingException,
                        exception: e,
                        message: "An error occurred stopping the application");
                }
            }
        }

        public void NotifyStarted()
        {
            try
            {
                _startedSource.Cancel();
            }
            catch (Exception e)
            {
                _logger.LogCritical(
                    eventId: LoggerEventIds.ApplicationStartupException,
                    exception: e,
                    message: "An error occurred starting the application");
            }
        }
        
        public void NotifyStopped()
        {
            try
            {
                _stoppedSource.Cancel();
            }
            catch (Exception e)
            {
                _logger.LogCritical(
                    eventId: LoggerEventIds.ApplicationStoppedException,
                    exception: e,
                    message: "An error occurred stopping the application");
            }
        }
    }
}